#ifndef GOOGLEREADER_H
#define GOOGLEREADER_H

#include <QObject>
#include <QString>
#include <QChar>
#include <QDateTime>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QList>
#include <QPair>
#include <QSslError>
#include <QFlags>

#include <subscriptions.h>
//class QNetworkReply;

class GoogleReader : public QObject
{
Q_OBJECT

public:
    enum Job {
        FetchFeeds,
        FetchFeed,
        SetRead
    };
    Q_DECLARE_FLAGS(Jobs, Job)


    explicit GoogleReader(QObject *parent = 0);
    void setUsername(QString user) { m_mail = user; }
    void setPassword(QString password) { m_pass = password; }
    void fetchFeeds();
    void fetchFeed(QString id);
private:
    void getSID();
    void getToken();
    QVariant _getCookie();
    void _fetchFeeds();
    void _fetchFeed(QString id, QDateTime timestamp = QDateTime(), QChar sort = 'd', int max = 100);
    void addJob(Job job, QVariant value = 0);
    void checkJobs();

    QNetworkAccessManager *m_manager;
    QNetworkReply *m_loginReply;
    QNetworkReply *m_tokenReply;
    QNetworkReply *m_fetchFeedsReply;
    QNetworkReply *m_fetchFeedReply;
    bool m_connected;
    bool m_isConnecting;
    bool m_activeJob;
    QString m_mail;
    QString m_pass;
    QByteArray m_sid;
    QByteArray m_auth;
    QList< QPair<Job, QVariant> > m_pendingJobs;

    Subscriptions *m_subscriptionList;

signals:
    void gotSid();
    void gotToken();
    void subscriptionsUpdated(Subscriptions *m_subscriptionList);

public slots:

private slots:
    void sidReply();
    void tokenReply();
    void fetchFeedsReply();
    void fetchFeedReply();
    void error(QNetworkReply::NetworkError error);
    void sslErrors(const QList<QSslError> & errors);

};

#endif // GOOGLEREADER_H
