#include <QMap>
#include <QtDebug>
#include <QInputDialog>

#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    m_reader = new GoogleReader();
#if 0
    m_reader->setUsername("");
    m_reader->setPassword("");
#else
    m_reader->setUsername(QInputDialog::getText(this, "Google Reader", "Enter your Google Mail address"));
    m_reader->setPassword(QInputDialog::getText(this, "Google Reader", "Enter your password", QLineEdit::Password));
#endif

    connect(m_reader, SIGNAL(subscriptionsUpdated(Subscriptions *)), this, SLOT(updateSubscriptions(Subscriptions*)));
    connect(ui->feedTree, SIGNAL(itemClicked(QTreeWidgetItem*,int)), this, SLOT(showFeed(QTreeWidgetItem*)));

    m_reader->fetchFeeds();
}

void MainWindow::updateSubscriptions(Subscriptions * subscriptions)
{
    ui->feedTree->clear();

    CategoryList categories = subscriptions->categories();
    for (int i = 0; i < categories.size(); ++i) {
        QString key = categories.at(i).first;
        QString value = categories.at(i).second;

        QTreeWidgetItem *categorieItem = new QTreeWidgetItem(ui->feedTree);
        categorieItem->setText(0, value);
        categorieItem->setToolTip(0, key);

        FeedList feeds = subscriptions->feedsForCategory(key);
        for (int k = 0; k < feeds.size(); ++k) {
            Feed f = feeds.at(k);
            QTreeWidgetItem *feedItem = new QTreeWidgetItem(categorieItem);
            feedItem->setText(0, f.title());
            feedItem->setToolTip(0, f.id());
        }
    }
}

void MainWindow::showFeed(QTreeWidgetItem *item)
{
    qDebug() << "clicked Feed " << item->toolTip(0);
    m_reader->fetchFeed(item->toolTip(0));
}

MainWindow::~MainWindow()
{
    delete ui;
}
