#include "subscriptions.h"

#include <QtDebug>
#include <QDomDocument>

class SubscriptionsXmlException {};

Subscriptions::Subscriptions(QObject *parent) :
    m_feeds()
{
    m_parent = parent;
}

void Subscriptions::read(const char *xml)
{
    m_feeds.clear();

    // see: http://code.google.com/p/google-reader-notifier-qt/source/browse/trunk/google-reader-notifier-qt/src/unreaditemscheck.cpp

    QDomDocument document;
    if (! document.setContent(QString::fromUtf8(xml)))
    {
        qWarning() << "could not read document!";
        return;
    }

    QDomNodeList tmpNodes;

    // the root element's name must be "object"
    QDomElement root = document.documentElement();
    if (root.tagName() != "object") throw SubscriptionsXmlException();

    // the root object must have _exactly_ one child element, named "list"
    tmpNodes = root.childNodes();
    if (tmpNodes.length() != 1) throw SubscriptionsXmlException();

    // the tag "list" must have "name" attr with value "subscriptions"
    QDomElement listElem = tmpNodes.at(0).toElement();
    if (listElem.isNull() || listElem.attribute("name") != "subscriptions") throw SubscriptionsXmlException();

    tmpNodes = listElem.childNodes();

    // parsing all "object" elements
    for (int i = 0; i < tmpNodes.size(); ++i)
    {
        QDomElement elem = tmpNodes.at(i).toElement();
        if (elem.isNull() ) throw SubscriptionsXmlException();

        Feed feed;
        //qDebug() << "feed tagName" << elem.tagName();


        QDomNodeList tmpNodes2 = elem.childNodes();
        for (int j = 0; j < tmpNodes2.size(); ++j)
        {
            QDomElement elem2 = tmpNodes2.at(j).toElement();
            if (elem2.tagName() == "list" && elem2.attribute("name") == "categories")
            {
                // Iterate over all categorie objects
                QDomNodeList tmpNodes3 = elem2.elementsByTagName("object");
                for (int k = 0; k < tmpNodes3.size(); ++k)
                {
                    QDomElement elem3 = tmpNodes3.at(k).toElement();

                    QString id = "";
                    QString label = "";

                    QDomNodeList tmpNodes4 = elem3.elementsByTagName("string");
                    for (int l = 0; l < tmpNodes4.size(); ++l)
                    {
                        QDomElement elem4 = tmpNodes4.at(l).toElement();
                        if (elem4.attribute("name") == "id") id = elem4.text();
                        else if (elem4.attribute("name") == "label") label = elem4.text();
                    }

                    feed.addCategorie(id, label);
                }
                //qDebug() << "add " << id << " " << label;
            } else if (elem2.tagName() == "string" && elem2.attribute("name") == "id")
            {
                feed.setId(elem2.text());
            } else if (elem2.tagName() == "string" && elem2.attribute("name") == "title")
            {
                feed.setTitle(elem2.text());
            }
        }

        m_feeds.append(feed);
    }
}

CategoryList Subscriptions::categories()
{
    CategoryList categories;

    for (int i = 0; i < m_feeds.size(); ++i) {
        Feed feed = m_feeds.at(i);
        QList< CategoryPair > cats = feed.categories();

        for (int j = 0; j < cats.size(); ++j) {
            if (categories.count(cats.at(j)) == 0)
                categories.append(cats.at(j));
        }
    }

    return categories;

}

FeedList Subscriptions::feedsForCategory(QString id)
{
    FeedList feeds;

    for (int i = 0; i < m_feeds.size(); ++i) {
        Feed feed = m_feeds.at(i);
        QList< QPair<QString, QString> > categories = feed.categories();
        for (int j = 0; j < categories.size(); ++j) {
            if (categories.at(j).first == id)
            {
                feeds.append(feed);
                break;
            }
        }
    }

    return feeds;
}

