/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) <year>  <name of author>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


#include "googlereaderconnection.h"
#include <krssresource/util.h>
#include <krss/rssitem.h>
#include <krss/item.h>

#include <KLocale>
#include <KDebug>
#include <QList>
#include <QByteArray>
#include <QLatin1String>

#include <qjson/parser.h>

GooglereaderConnection::GooglereaderConnection( QObject *parent ) :
    QObject(parent),
    m_connected(false),
    m_isConnecting(false),
    m_activeJob(false),
    m_mail(),
    m_pass(),
    m_sid(),
    m_token(),
    m_uid()
{

}

void GooglereaderConnection::login()
{
   kWarning() << "logging in!";
   if (!m_connected && !m_isConnecting)
     getSID();
   else if (m_connected)
     emit loggedIn();
}


void GooglereaderConnection::getSID()
{
    kWarning() << "request sid...";

    m_isConnecting = true;

    KUrl url( "https://www.google.com/accounts/ClientLogin" );
    url.addQueryItem(QLatin1String("service"), QLatin1String("reader"));
    url.addQueryItem(QLatin1String("Email"), m_mail);
    url.addQueryItem(QLatin1String("Passwd"), m_pass);
    url.addQueryItem(QLatin1String("source"), QLatin1String("Akonadi Google-Reader RSS resource"));
    url.addQueryItem(QLatin1String("continue"), QLatin1String("http://www.google.com/"));

    KIO::TransferJob* job = KIO::http_post( url, url.encodedQuery(), KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));

    kWarning() << "Buffer:" << url.encodedQuery();
    
    connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
             this, SLOT( slotSidData( KIO::Job*, const QByteArray& ) ) );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotSidResult( KJob* ) ) );
}

void GooglereaderConnection::slotSidData( KIO::Job *job, const QByteArray& data )
{
    Q_UNUSED( job )
    kWarning() << "Got sid data:" << data;
    
    QList<QByteArray> dataArray = data.split('\n');
    
    for (int i = 0; i < dataArray.size(); ++i) {
        if (dataArray.at(i).startsWith("SID="))
        {
            kWarning() << "Found SID:" << dataArray.at(i).mid(4);
            m_sid = dataArray.at(i).mid(4);
        }
        else if (dataArray.at(i).startsWith("Auth="))
        {
            kWarning() << "Found Auth:" << dataArray.at(i).mid(5);
            m_auth = dataArray.at(i).mid(5);
        }

     }
}

void GooglereaderConnection::slotSidResult( KJob *job )
{
    kWarning() << "slotSidResult";
    if ( job->error() ) {
        kWarning() << job->errorString();
        //setError( CouldNotContactNewsgator );
        //setErrorText( job->errorString() );
        //emitResult();
        return;
    }

    //m_soapMessage->parse();
    /*if ( m_soapMessage->hasError() ) {
        kWarning() << m_soapMessage->errorString();
        //setError( CouldNotParseSoapMessage );
        //setErrorText( m_soapMessage->errorString() );
        //emitResult();
        return;
    }

    if ( m_soapMessage->messageType() == SoapMessage::Fault ) {
        kWarning() << m_soapMessage->soapErrorString();
        setError( SoapRequestFailed );
        setErrorText( m_soapMessage->soapErrorString() );
        emitResult();
        return;
    }*/

    //emitResult();

    getToken();
}

void GooglereaderConnection::getToken()
{
    kWarning() << "request token...";

    KUrl url( "http://www.google.com/reader/api/0/token" );

    KIO::TransferJob* job = KIO::get( url, KIO::Reload, KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));
    job->addMetaData(QLatin1String("customHTTPHeader"), QLatin1String("Authorization: GoogleLogin auth="+ authorization()));

    connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
             this, SLOT( slotTokenData( KIO::Job*, const QByteArray& ) ) );
    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotTokenResult( KJob* ) ) );
}

void GooglereaderConnection::slotTokenData( KIO::Job *job, const QByteArray& data )
{
    Q_UNUSED( job )
    kWarning() << "Got token data:" << data;
    if (data.length() != 0)
      m_token = data;

}

void GooglereaderConnection::slotTokenResult( KJob *job )
{
    kWarning() << "slotTokenResult";

    getUserdata();
    //emit loggedIn();
}

void GooglereaderConnection::getUserdata()
{
    kWarning() << "request userdata...";

    KUrl url( "http://www.google.com/reader/api/0/user-info" );

    KIO::StoredTransferJob * const job = KIO::storedGet( url, KIO::Reload, KIO::HideProgressInfo );
    job->addMetaData(QLatin1String("UserAgent"), QLatin1String("Google-Reader Akonadi News fetcher"));
    job->addMetaData(QLatin1String("customHTTPHeader"), QLatin1String("Authorization: GoogleLogin auth="+ authorization()));

    connect( job, SIGNAL( result( KJob* ) ), this, SLOT( slotUserResult( KJob* ) ) );
}


void GooglereaderConnection::slotUserResult( KJob *job )
{
    const KIO::StoredTransferJob * const sjob = qobject_cast<const KIO::StoredTransferJob*>( job );
    Q_ASSERT( sjob );
    QByteArray data = sjob->data();
    
    kWarning() << "Got user data:" << data;

    QJson::Parser parser;
    bool ok;

    QVariantMap result = parser.parse(data, &ok).toMap();
    if (ok)
    {
        m_uid = result[QLatin1String("userId")].toString();
        kWarning() << "user is is" << m_uid;
    }

    emit loggedIn();
}
