/*
    Copyright (C) 2009    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef KRSS_GOOGLEREADERRESOURCE_GOOGLEREADERJOBS_H
#define KRSS_GOOGLEREADERRESOURCE_GOOGLEREADERJOBS_H

//#include "location.h"
#include <googlereaderconnection.h>

#include <krssresource/rssbackendjobs.h>
#include <krss/feedcollection.h>

#include <Syndication/Syndication>
#include <Syndication/DataRetriever>
#include <Akonadi/Collection>
#include <KJob>
#include <QtCore/QString>
#include <boost/graph/graph_concepts.hpp>

namespace KIO {
class Job;
}

class KJob;
class QByteArray;

class GooglereaderFeedsRetrieveJob : public KRssResource::FeedsRetrieveJob
{
    Q_OBJECT
public:
    explicit GooglereaderFeedsRetrieveJob( GooglereaderConnection *connection, QObject *parent = 0 );

    QList<Akonadi::Collection> feeds() const;

    QString errorString() const;
    void start();

private Q_SLOTS:
    void slotData( KIO::Job *job, const QByteArray& data );
    void slotResult( KJob *job );
    void loggedIn();

private:
    GooglereaderConnection *m_connection;
    QByteArray m_data;
    QList<Akonadi::Collection> m_feeds;
    Q_DISABLE_COPY( GooglereaderFeedsRetrieveJob )
};

class GooglereaderFeedCreateJob : public KRssResource::FeedCreateJob
{
    Q_OBJECT
public:
    explicit GooglereaderFeedCreateJob( GooglereaderConnection *connection, QObject *parent = 0 );

    Akonadi::Collection feed() const;
    void setXmlUrl( const QString& xmlUrl );

    QString errorString() const;
    void start();

private Q_SLOTS:
    void loggedIn();
    void slotResult( KJob *job );
    void slotFeedsRetrieved( KJob *job );

private:
    GooglereaderConnection *m_connection;
    QString m_xmlUrl;
    int m_subscriptionId;
    KRss::FeedCollection m_feed;
    Q_DISABLE_COPY( GooglereaderFeedCreateJob )
};

class GooglereaderFeedModifyJob : public KRssResource::FeedModifyJob
{
    Q_OBJECT
public:
    explicit GooglereaderFeedModifyJob( GooglereaderConnection *connection, QObject *parent = 0 );

    void setFeed( const Akonadi::Collection& feed );
    Akonadi::Collection feed() const;

    QString errorString() const;
    void start();

private Q_SLOTS:
    void loggedIn();
    void slotResult( KJob *job );

private:
    GooglereaderConnection *m_connection;
    KRss::FeedCollection m_feed;
    Q_DISABLE_COPY( GooglereaderFeedModifyJob )
};

class GooglereaderFeedDeleteJob : public KRssResource::FeedDeleteJob
{
    Q_OBJECT
public:
    explicit GooglereaderFeedDeleteJob( GooglereaderConnection *connection, QObject *parent = 0 );

    void setFeed( const Akonadi::Collection& feed );
    Akonadi::Collection feed() const;

    QString errorString() const;
    void start();

private Q_SLOTS:
    void loggedIn();
    void slotResult( KJob *job );

private:
    GooglereaderConnection *m_connection;
    KRss::FeedCollection m_feed;
    Q_DISABLE_COPY( GooglereaderFeedDeleteJob )
};

class GooglereaderFeedFetchJob : public KRssResource::FeedFetchJob
{
    Q_OBJECT
public:
    explicit GooglereaderFeedFetchJob( GooglereaderConnection *connection, QObject *parent = 0 );

    void setFeed( const Akonadi::Collection& feed );
    Akonadi::Collection feed() const;
    QList<Akonadi::Item> items() const;

    void start();
    QString errorString() const;

private Q_SLOTS:
    void loggedIn();
    void slotResult( KJob *job );

private:
    GooglereaderConnection *m_connection;
    KRss::FeedCollection m_feed;
    QList<Akonadi::Item> m_items;
    Q_DISABLE_COPY( GooglereaderFeedFetchJob )
};

class GooglereaderItemModifyJob : public KRssResource::ItemModifyJob
{
    Q_OBJECT
public:
    explicit GooglereaderItemModifyJob( QObject *parent = 0 );

    void setItem( const Akonadi::Item& item );
    Akonadi::Item item() const;

    QString errorString() const;
    void start();

private Q_SLOTS:
    void slotResult( KJob *job );

private:
    Akonadi::Item m_item;
    Q_DISABLE_COPY( GooglereaderItemModifyJob )
};

#endif // KRSS_GOOGLEREADERRESOURCE_GOOGLEREADERJOBS_H

