/*
    Copyright (C) 2009    Dmitry Ivanov <vonami@gmail.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "googlereaderrssresource.h"
#include "settingsdialog.h"
#include "googlereadersettings.h"
#include "googlereaderjobs.h"

#include <KDebug>

using namespace KRss;
using namespace KRssResource;

GooglereaderRssResource::GooglereaderRssResource( const QString& id )
    : RssResourceBase( id )
{
    connect( this, SIGNAL( reloadConfiguration() ), this, SLOT( slotReloadConfiguration() ) );
    loadConfiguration();
}

void GooglereaderRssResource::configure( WId windowId )
{
    Q_UNUSED( windowId );

    kWarning() << "configure...";

    SettingsDialog * const dialog = new SettingsDialog();
    dialog->setPassword(Settings::password());
    dialog->setUserName(Settings::userName());
    if ( dialog->exec() ) {
        //kDebug() << "Location:" << dialog->location().name << ", id:" << dialog->location().id;
        m_userName = dialog->userName();
        m_password = dialog->password();
        //m_location = dialog->location();
        Settings::setUserName( dialog->userName() );
        Settings::setPassword( dialog->password() );
        //Settings::setLocationName( dialog->location().name );
        //Settings::setLocationId( dialog->location().id );
        
        Settings::self()->writeConfig();
        synchronizeCollectionTree();
    }
    delete dialog;
}

void GooglereaderRssResource::slotReloadConfiguration()
{
    kWarning() << "reload configuration";
    loadConfiguration();
    synchronizeCollectionTree();
}

void GooglereaderRssResource::loadConfiguration()
{
    kWarning() << "load configuration";
    m_userName = Settings::userName();
    m_password = Settings::password();
    //m_location.name = Settings::locationName();
    //m_location.id = Settings::locationId();
    //kDebug() << "Loaded location:" << m_location.name << ", id:" << m_location.id;

    m_connection = new GooglereaderConnection();
    m_connection->setUsername(Settings::userName());
    m_connection->setPassword(Settings::password());

    kWarning() << "username:" << Settings::userName();
    kWarning() << "password:" << Settings::password();
}

bool GooglereaderRssResource::flagsSynchronizable() const
{
    return true;
}

FeedsRetrieveJob* GooglereaderRssResource::feedsRetrieveJob() const
{
    GooglereaderFeedsRetrieveJob *job = new GooglereaderFeedsRetrieveJob(m_connection);
    //job->setUserName( m_userName );
    //job->setPassword( m_password );
    //job->setLocation( m_location );
    return job;
}

FeedsImportJob* GooglereaderRssResource::feedsImportJob() const
{
    return 0;
}

FeedCreateJob* GooglereaderRssResource::feedCreateJob() const
{
    GooglereaderFeedCreateJob *job = new GooglereaderFeedCreateJob(m_connection);
    //job->setLocation( m_location );
    return job;
}

FeedModifyJob* GooglereaderRssResource::feedModifyJob() const
{
    GooglereaderFeedModifyJob *job = new GooglereaderFeedModifyJob(m_connection);
    //job->setLocation( m_location );
    return job;
}

FeedDeleteJob* GooglereaderRssResource::feedDeleteJob() const
{
    GooglereaderFeedDeleteJob *job = new GooglereaderFeedDeleteJob(m_connection);
    //job->setLocation( m_location );
    return job;
}

FeedFetchJob* GooglereaderRssResource::feedFetchJob() const
{
    GooglereaderFeedFetchJob *job = new GooglereaderFeedFetchJob(m_connection);
    //job->setLocation( m_location );
    return job;
}

KRssResource::ItemModifyJob* GooglereaderRssResource::itemModifyJob() const
{
    GooglereaderItemModifyJob *job = new GooglereaderItemModifyJob();
    //job->setLocation( m_location );
    return job;
}

AKONADI_RESOURCE_MAIN( GooglereaderRssResource )

#include "googlereaderrssresource.moc"
